<?php

/**
 * go() sends someone to the selected image and then stops execution
 * of the script.
 */
function go($image, $title){
	global $debug;

	if($debug){
		print $image;
	} else {
		header("Location: https://apps.library.und.edu/stack-maps/map.php?image=$image&title=$title");
	}

	exit;
}

/**
 * normalize_lc normalizes a Library of Congress call number
 *
 * @param $lc The Library of Congress call number to be normalized, e.g., HQ 1438.N35 U47 1991
 * @return The normalized call number, e.g., HQ 1438.0000N 3500U 4700 1991
 */
function normalize_lc($lc){
    $matches = array();

    // We stole this from http://robotlibrarian.billdueber.com/2008/11/normalizing-loc-call-numbers-for-sorting/

    $pattern = "/^
              \s*
              ([A-Z]{1,3})  # alpha
              \s*
              (         # optional numbers
                \d+
                (?: \s*\.\s*\d+)?  # ...with optional decimal point
              )?
              \s*
              (?:               # optional cutter
                \.? \s*
                ([A-Z]+)      # cutter letter
                \s*
                (\d+)?        # cutter numbers
              )?
              (?:               # optional cutter
                \.? \s*
                ([A-Z]+)      # cutter letter
                \s*
                (\d+)?        # cutter numbers
              )?
              (?:               # optional cutter
                \.? \s*
                ([A-Z]+)      # cutter letter
                \s*
                (\d+)?        # cutter numbers
              )?
              \s*
              (.*?)            # everthing else
              \s*$
            /x";

    preg_match($pattern, $lc, $matches);

    list($lc, $alpha, $num, $c1alpha, $c1num, $c2alpha, $c2num, $c3alpha, $c3num, $extra) = $matches;
    $c1num = str_pad($c1num, 4, "0");
    $c2num = str_pad($c2num, 4, "0");
    $c3num = str_pad($c3num, 4, "0");
    if(!empty($extra)){ $extra = " $extra"; }

    return sprintf("%-3s%09.4f%-2s%4s%-2s%4s%-2s%4s%s", $alpha, $num, $c1alpha, $c1num, $c2alpha, $c2num, $c3alpha, $c3num, $extra);
}

/**
 * check_location() attempts to pick a stack map based on the location code.
 * It returns either an image file name or boolean false if it's an item in the general stacks.
 */
function check_location($location_code){

	switch($location_code){
		case "GOVDO": // Gov Docs oversize
		case "POST1":
		case "GOVD":
		case "GOVDJ":
			$image = "cfl-01-government.png";
		break;

		case "MAPS":
		case "STATM":
			$image = "cfl-01-maps.png";
		break;

		case "OVER":
			$image = "cfl-01-oversized.png";
		break;

		case "GOVDM":
		case "GOVOF":
		case "GOVDS":
		case "AV1":
		case "AVKIT":
		case "MCRD":
		case "MFE":
		case "MFEJ":
		case "MFM":
		case "MFMJ":
		case "MPRT":
		case "PDOFC":
		case "SOFT1":
		case "ARMFE":
		case "AUDCD":
		case "AUDI1":
		case "SOFCD":
		case "TVIDE":
		case "AVVIS":
			$image = "cfl-01-periodicalsoff.png";
		break;

		case "NEWSP":
			$image = "cfl-01-newspapers.png";
		break;

		case "JLS1":
		case "OVRJ1":
			$image = "cfl-01-periodicals.png";
		break;

		case "UIT":
			$image = "cfl-01-uit.png";
		break;

		case "STAT4":
		case "STATO":
			$image = "cfl-01-state.png";
		break;

		case "RESRV":
		case "ILL":
		case "CIRC2":
			$image = "cfl-02-access.png";
		break;

		case "REFD":
		case "PAT":
			$image = "cfl-02-infodesk.png";
		break;

		case "REF":
			$image = "cfl-02-reference.png";
		break;

		case "POPR":
			$image = "cfl-02-popular.png";
		break;

		case "GAMES":
			$image = "cfl-03-wboard.png";
		break;

		case "AVVID":
			$image = "cfl-03-wdvd.png";
		break;

		case "CHDRN":
		case "YDULT":
			$image = "cfl-04-children.png";
		break;

		// Books pulled from general collection for storage
		// in compact shelving.
		case "CSTOR":
			$image = "cfl-01-cstor.png";
		break;


		// The general stacks on floor 3 and 4 need
		// to be handled by call number, so we
		// leave image as false.
		case "CFL3":
		case "CFL4":
		case "ELL":
			$image = false;
		break;

		// In case of an unrecognized code
		default:
			$image = "cfl-default.png";
		break;
	}

	return $image;
}


/**
 * normalize_ranges() normalizes the ranges of the stacks.
 */
function normalize_ranges($ranges){
	$normalized = [];

	// Loop through the list of ranges and normalize them.
	foreach($ranges as $lc => $image){
		$cur = normalize_lc($lc);
		$normalized[$cur] = $image;
	}

	return $normalized;
}


/**
 * find_image() takes a call number in the main stacks and returns
 * either the appropriate image or an array of debug info if it couldn't
 * find one.
 */
function find_image($call_number, $ranges){
/*
The basic approach to figuring out what shelf image
to use is as follows:

1. We have an array of call numbers.  They define the
start and end of each shelf range.

2. We normalize the call numbers so that the computer
can sort them and arrive at the correct order.

3. We normalize the call number of the current item.

4. If it exactly matches any of the numbers in the range
because this particular item happens to fall at the
beginning or end of a range, then we return that image.

5. If it does NOT match any of them exactly, then we
add that number to the array of call numbers and sort it.

6. We loop through the array and record the
shelf image before and after it.

7. The "before" and "after" images should be identical. If they're not, something went wrong (maybe a typo in the call number in Alma?) and we go to the default image.

8. If the before and after images match, then we send that shelf image.
*/




	global $debug;
	$results = [
		'call_number' => $call_number,
	];

	// Normalize the book's call number.
	$call_number = normalize_lc($call_number);

	$results['normalized_call_number'] = $call_number;

	// If someone has a book exactly at the beginning
	// or end of a range, that call number will already
	// be in the array.  If so, send them the associated image
	// and exit the script.
	if(array_key_exists($call_number, $ranges)){
		if($debug){ print "Location matches an end cap.\n"; }
		return $ranges[$call_number];
	}

	// Add the normalized number to the array.
	$ranges[$call_number] = "";

	// Sort the array by key.
	ksort($ranges);

	// Loop through the array.
	foreach($ranges as $lc => $image){
		if($image === ""){
			// The current entry is the one we're looking for.
			// Log the "previous" variable to results.
			$results['previous'] = $previous;
		} else {
			if(isset($results['previous'])){
				// The current entry is the one immediately
				// after the one we're looking for.
				// Log it as the "next" variable.
				$results['next'] = $image;
				break;
			} else {
				// We have not found the item we're looking for
				// yet.  Update the $previous variable to record
				// the one we're looking at now.
				$previous = $image;
			}
		}
	}

	if($results['previous'] != $results['next']){
		// We're having trouble identifying the shelf.
		// The two don't match.
		// Go to the default image.
		if($debug){
			print "Unable to determine image.\n";
			print "Normalized call number was: $call_number\n";
			print "Before and after values were:\n";
			var_dump($results);
			print "\n";
		}
		return $results;
	}

	return $results['previous'];
}
